# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2004,2006 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# $Id: _nlsmap.awk,v 1.2 2006/03/10 21:36:17 carlo Exp $
# =====================================================================
# string _nlsmap(string name, string value \
#			[, string context, string repl, int opt])
#
# Takes a name/value pair and looks for the corresponding NLS
# translation. If lookup fails returns the original value unchanged. If
# 'context' is not specified it defaults to the null value, otherwise
# it can be almost any string, often the calling program name.
# Occasionally, the same program may want to use multiple output forms,
# each one with its own mapping set. In those cases the program may
# want to call this function passing the template name, rather than the
# program name, in the 'context' argument. Of course the local _NLSMAP
# will need to have been built accordingly. As said, 'context' can
# be any string, even something like "dir/template", so this scheme
# turns out to be very flexible. The optional 'repl' parameter is a
# substitution token for those map entries that take a parameter in
# the form of a single '%s'. If the 'opt' flag is set to _O_CURR, then
# the result of the lookup will be returned in currency format, where
# applicable.

# =====================================================================

function _nlsmap(name,value,context,repl,opt,		fmt) {

  # Now a null pgm value indicates a generic map lookup.

  #if (context == _NULL) {
  #   context = ENVIRON["CSA_PGM"]
  #   sub(/.*\001/,_NULL,context)	# keep only rightmost value.
  #}

  if (_NLSMAP[context,name,value] == _NULL) {

     if (opt == _O_CURR) {

	# strip leading '$' sign added by PostgreSQL on currency values.
	sub(/^\$/,"",value)

	# Look-up the format string and make sure it is not empty,
	# or things may run wild.

	if ((fmt="%s" _nlsmap("currfmt")) == _NULL) fmt = "%.2f"
	return sprintf(fmt,_nlsmap("currency",_NULL,_NULL,repl),value)
     }

     return value
  }

  if (repl != _NULL) return sprintf(_NLSMAP[context,name,value],repl)

  return _NLSMAP[context,name,value]
}

