# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2004,2006 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# $Id: _sendmail.awk,v 1.2 2006/03/10 21:36:17 carlo Exp $

# =====================================================================
# int _sendmail(string template [,string eval,boolean trim,int lvl])
#
# Send an e-mail message from within a _userproc(_O_RESPONSE) context,
# based on a message template file.
# =====================================================================

function _sendmail(tpl,eval,trim,lvl,		      		      \
	   		ltago,ltagc,retago,retagc,nl,end,tlen,tag,enc,\
			a,i,j,input,record,start,tago,tagc) {

  i=j=0

  # Check args and set defaults.
  if (tpl == _NULL) return(1)				# no default!
  if (tpl !~ /^\//)
     tpl = ENVIRON["CSA_ROOT"] "/forms/" ENVIRON["CSA_LANG"] "/" tpl
  tago = "$["						# fixed.
  tagc = "]"						# fixed.
  ltago = 2						# fixed.
  ltagc = 1						# fixed.

  if (!ENVIRON["CSA_TPLMAX"]) ENVIRON["CSA_TPLMAX"] = 10

  # prevent infinite recursions.
  if (lvl > ENVIRON["CSA_TPLMAX"]) return 1

  # Prepare the substitution regexp's.
  retago = _escreg(tago)
  retagc = _escreg(tagc)

  retago = retago "[-A-Za-z0-9_/.]+" retagc

  if (!lvl) {				# only if 1st recursion.

     # being this function fundamentally an extension of the HTTP
     # response template processing, the relevant substitution
     # variables are obtained by reverse-encoding their corresponding
     # XML_ versions.

     for (j in ENVIRON) {

	 if (!sub(/^XML_/,"",j) || j == _NULL) continue

	 ENVIRON[j] = _xmldecode(ENVIRON["XML_" j])
     }

     if (_CMD != _NULL) close(_CMD)		# just in case.

     _CMD = ENVIRON["CSA_CMD_SENDMAIL_"]	# open MTA command.
  }

  if (eval != "") split(eval,a,",")

  # Process the template file.

  while (getline input < tpl > 0) {
     if (input ~ /^#/) continue	    		# skip comment lines.
     nl = 1
     do {
        match(input,retago)
        if (RSTART) {
	   end = index(input, tagc)
	   tlen = end - RSTART - ltago
	   start = RSTART
	   printf("%s",substr(input, 1, start - 1)) | _CMD
	   tag = substr(input, start + ltago, tlen)

	   # Since AWK can open only one _CMD at a time, no system()
	   # are allowed here, so no encoding flags are supported in
	   # file tags. On the other hand it is trivial for the calling
	   # program to prepare pre-encoded binary attachments with
	   # 'csa-encdec -e -b' or something to that effect.

	   for (i in a) {			# Is "tag" to be eval'ed ?
	       if (a[i] == tag) {
		  tag = ENVIRON[tag]		# Get tag value from env.
		  break
	       }
	   }

	   if (tag ~ /\//) {			# Catch file patterns.
	      # Catch any '..' pattern and make sure tag is in full-path.
	      if (tag ~ /\.\./ || tag !~ /^\//) {
		 printf("%s%s%s",tago,tag,tagc) | _CMD	# Print tag literally.
	      }

	      # any binary stuff is required to have already been
	      # b64-encoded by the caller.

	      else {
		 gsub(/\/\//, "/", tag)		# Turn any "//" into "/".

		 # include the specified content.
		 _sendmail(tag,eval,trim,lvl+1)
		 nl = 0
	      }
	   }
	   else {
	      if (trim) ENVIRON[tag] = _strip(ENVIRON[tag])
	      printf("%s",ENVIRON[tag]) | _CMD
	   }

	   input = substr(input, end + ltagc)
	}
	else {
	   printf("%s",input) | _CMD
	   input = _NULL		# Not to loop forever.
	}
     }  while (length(input))

     if (nl) printf("\n") | _CMD
  }

  close(tpl)				# close input.

  if (!lvl) close(_CMD)			# send message if at top level.
}

